 function [lossf,gap,Pcal,Piden,yhat]=clvalid(B,A,R,S,T,y,r,pcl)
%	CLVALID	is used for validation of the plant model in closed loop.
%			
%	[lossf,gap,Pcal,Piden,yhat]=clvalid(B,A,R,S,T,y,r,pcl)
%
%	The estimation of the cross-correlation function between the 
%	prediction error and the estimated output together with a 
%	confidence interval of %97 will be given. 
%	The plant output y and the estimated one are compared and the loss 
%	function is computed.
%			
%	B and A are respectievely the numerator and denominator of the plant 
%	model to be validated.
%
%	R, S and T are the column vectors containing the parameters of a two
%	degree of freedom controller. S*u(t)=-R*y(t)+T*r(t)
% 	Remark: when the excitation signal is added to the measured output 
%	(i.e. the controller is in feedforward with unit feedback) we have T=R and  
%	when the excitation signal is added to the control input (i.e. the controller
%	is in feedback) we have T=S.
%
%	y and r are the column vectors of output and excitation signal in
%	closed loop operation.
%
%	if pcl=1 a pole closeness validation is also performed (pcl=0 by default).
%	In this validation test the identified closed loop poles using ARMAX
%	method are compared with the computed closed loop poles (using the model
%	and the controller).
%						
%
%	lossf : Loss function (the sum of (y-yhat)^2 divided by number of data)
%	gap	: Vinnicombe gap metic between identified closed loop transfer function
%			  and computed transfer function. 
%	Pcal  : Computed closed loop poles by given model and controller
%	Piden : Identified closed loop poles from [y r] data
%	yhat  : Closed loop estimated output
%
%	Copyright by Adaptech, 1997-1999
%	Last modified on November 10, 2004 
%
%  The source code is available from Adaptech 
%  
%  Contacts:
%  
%  Adaptech
%  4 rue du Tour de l'Eau
%  38400 St Martin d'Hres 
%  France
%  
%  Tel :  +33 4 76 51 52 77  
%  Fax :  +33 4 76 42 84 16  
%  Email : info@adaptech.com
%  Web site: www.adaptech.com


na=length(A);
nB=length(B);
nr=length(R);
ns=length(S);
nt=length(T);

[nl,nc]=size(y);
if nc>2, error('This routine is only for SISO systems'),end
[nl,nc]=size(r);
if nc>2, error('This routine is only for SISO systems'),end

R=R./S(1);T=T./S(1);S=S./S(1);
[rs,cs]=size(S);
[rr,cr]=size(R);
[rt,ct]=size(T);
if min(size(R))>1, error('R should be a column vector!');end
if min(size(S))>1, error('S should be a column vector!');end
if min(size(T))>1, error('T should be a column vector!');end
if cs>rs, S=S';end
if cr>rr, R=R';end
if ct>rt, T=T';end

d=-1;i=1;
while B(i)==0,d=d+1;i=i+1;end
if d==-1, error('B should contain at least one zero!');end
if d==nB-1, error('B should be a nonzero vector!');end
nb=nB-d-1;

nd=min(length(r),length(y));	% number of data

if nargin<8, pcl=0;end


	nas=na+ns-1;
	nbr=nr+nb+d;
	if nas>nbr, zas=0;zbr=nas-nbr;else zas=nbr-nas;zbr=0;end
	AS=conv(A,S);BR=conv(B,R);
	if max(size(S))==1, AS=AS';end
   if max(size(R))==1, BR=BR';end
% **********************************   
   if size(AS,2)~=1 AS=AS'; end;
   if size(BR,2)~=1 BR=BR'; end;
% **********************************   
	Pcal=[AS;zeros(zas,1)]+[BR;zeros(zbr,1)];
	np=length(Pcal);
	BT=conv(B,T);
	yhat=filter(BT',Pcal',r);

    th_lim=num2str(2.17/sqrt(nd));
    
if pcl~=0,
   % Identification of the closed loop
   [BTiden,Piden,Hiden]=xcloe(y,r,np-1,nb+nt-1,np-1,d,0,1,1);

   % Calculated and identified closed Loop frequency responses 
  
     bodenorm(BT,Pcal,BTiden,Piden,1000);
     
   % Vinnicombe gap
   
   gap=vgap(BT,Pcal,BTiden,Piden);
   
   % whiteness test for open loop identification of the closed loop
   [yhat_bf,err,stab]=modelsim(BTiden,Piden,Hiden,d,y,r,nd);
   
   
   if (stab)
       
       % max number of correlation terms
       nc=min(np,10);
       e0=abs(err'*err/nd);
       norme=e0;
       
       for i=1:nc
           ei(i)=err(1+i:nd)'*err(1:nd-i)/(nd-i);
           eni(i)=abs(ei(i)/norme);
       end
       
       % Warning if max(ENI) > 0.15 
       
       if (max(eni)>0.15)
           beep;
           disp('WARNING!');
           disp('The identified closed loop model is not validated');
       end
       
       figure;
       x1=1:nc;
       xx=0;yy=0;
       for i=1:nc,xx=[xx x1(i) x1(i) x1(i)];yy=[yy 0 eni(i) 0];end
       xx=[xx nc+1];yy=[yy 0];
       plot(xx,yy,0:nc+1,2.17/sqrt(nd)*ones(1,length(0:nc+1)),0:nc+1,0.15*ones(1,length(0:nc+1)),'-')
           hold on
       xlabel('i');
       ylabel('EN(i)');
       title('Whiteness test for OL identification of the closed loop')
       legend('Correlation terms',strcat('Theoretical limit: ',th_lim),'Practical limit: 0.15',1)
       figure;
       gapstr=num2str(gap);
       uicontrol(gcf,...
           'Units','points', ...
           'HorizontalAlignment','center', ...
           'ListboxTop',0, ...
           'BackgroundColor',get(gcf,'Color'), ...
           'Position',[123.25 242.75 200 33], ...
           'String','Vinnicombe Gap', ...
           'Style','text',...
           'Fontsize',20)
       uicontrol(gcf,...
           'Units','points', ...
           'HorizontalAlignment','center', ...
           'ListboxTop',0, ...
           'BackgroundColor',get(gcf,'Color'), ...
           'Position',[123.25 142.75 200 33], ...
           'String',[gapstr], ...
           'Style','text',...
           'Fontsize',20)
   end
else
	Piden=[];gap=[];
end



% error obtained
err=y-yhat;
nc=min(np,10);
r0=abs(err'*yhat/nd);
lossf=err'*err/nd;
normr=sqrt((yhat'*yhat/nd)*(lossf));

for i=1:nc
ri(i)=err(1+i:nd)'*yhat(1:nd-i)/(nd-i);
ei(i)=err(1+i:nd)'*err(1:nd-i)/(nd-i);
rni(i)=abs(ri(i)/normr);
end


   
   

disp('R0:');r0,
disp('RNi:');rni,
disp('max(RNi): ');max(rni)

% Warning if max(RNI) > 0.15 
if (max(rni)>0.15)
       beep;
       disp('WARNING!')
       disp('The identified plant model is not validated')
end
disp('gap : ');gap




figure;
x1=1:nc;
xx=0;yy=0;
for i=1:nc,xx=[xx x1(i) x1(i) x1(i)];yy=[yy 0 rni(i) 0];end
xx=[xx nc+1];yy=[yy 0];
%if pcl==0,subplot(212);else,subplot(223);end
plot(xx,yy,0:nc+1,2.17/sqrt(nd)*ones(1,length(0:nc+1)),0:nc+1,0.15*ones(1,length(0:nc+1)),'-')
xlabel('i');
ylabel('RN(i)');
title('Uncorrelation test of error and yhat')
legend('Correlation terms',strcat('Theoretical limit: ',th_lim),'Practical limit: 0.15',1)

figure;
plot([y,yhat]);
title(['Real and estimated Plant Output (loss function = ',num2str(lossf),')']);
xlabel('Sample')
if pcl~=0,
    figure;
    plot(roots(Piden),'o');zgrid
    hold;plot(roots(Pcal),'x');hold
    axis([-1 1 -1 1])
    title('o: identified poles, x: computed poles')
end
zoom

function [yhat,err,stab]=modelsim(B,A,Hbf,d,y,r,nd)

stab=1;
na=length(A)-1;
nb=length(B)-1-d;
nh_bf=length(Hbf)-1;

if(nh_bf>na)
    H=[A(2:end) zeros(nh_bf-na,1)] + Hbf(2:end);
elseif(nh_bf<na)
    H=A(2:end) + [Hbf(2:end) zeros(na-nh_bf,1)];
elseif(nh_bf==na)
    H=A(2:end) + Hbf(2:end);
end

if (abs(real(roots(H)))>1)
    stab=0;
    return
end

nh=length(H);
nth=na+nb+nh;
np=max([na+1,nb+d,nh]);

i=[1:np-1 np+1:2*np-1 2*np+1:3*np-1];           %shift index for vector phi
j=[1:na np+d+1:np+d+nb 2*np+1:2*np+nh];                 %phi index for theta

theta=[A(2:end) B(2+d:end) H]'; 
phi=zeros(3*np,1);

for t=1:nd
        yhat(t)=theta'*phi(j);
        err(t)=y(t)-yhat(t);
        phi(i+1)=phi(i);phi(1)=-y(t);phi(np+1)=r(t);phi(2*np+1)=err(t);
end

err=err';

function [mag,w]=bodenorm(B_cal,A_cal,B_iden,A_iden,N)

w=1:N/2;
z=exp(-i*2*pi*w/N);

num_cal=polyval(B_cal,z);
den_cal=polyval(A_cal,z);
mag_cal=abs(num_cal)./abs(den_cal);
mag_cal=20*log10(mag_cal);

num_iden=polyval(B_iden,z);
den_iden=polyval(A_iden,z);
mag_iden=abs(num_iden)./abs(den_iden);
mag_iden=20*log10(mag_iden);

w=(w/N);

plot(w,mag_cal)
hold on
plot(w,mag_iden,'r')
title('Magnitude Bode diagrams comparison')
legend('Calculated Closed Loop','Identified Closed loop',1)
set(gca,'XTick',[0.05 0.1 0.15 0.2 0.25 0.30 0.35 0.40 0.45 0.5])
set(gca,'XTickLabel',{'0.05', '0.10', '0.15', '0.20', '0.25', '0.30', '0.35', '0.40','0.45', '0.5'})
xlabel('Normalized frequencies (f/fs)');ylabel('Magnitude (dB)');
grid on