function [B,A]=fcloe(y,r,na,nb,d,R,S,T,B0,A0,Fin,lam1,lam0)
% FCLOE	is used to identify a discrete time model of a plant operating in 
%		closed-loop with an RST controller based on the F-CLOE method.
%
%		[B,A]=fcloe(y,r,na,nb,d,R,S,T,B0,A0,Fin,lam1,lam0)
%               
%		y and r are the column vectors containing respectively the output and the 
%		excitation signal.
%
%		na, nb are the order of the polynomials A,B and d is the pure time delay.
%
%		R, S and T are the column vectors containing the parameters of a two
%		degree of freedom controller. S*u(t)=-R*y(t)+T*r(t)
% 		Remark: when the excitation signal is added to the measured output 
%		(i.e. the controller is in feedforward with unit feedback) we have T=R and  
%		when the excitation signal is added to the control input (i.e. the controller
%		is in feedback) we have T=S.
%	
%		B0 and A0 are the numerator and denominator of an initial model.
% 
%		Fin is the initial gain F0=Fin*(na+nb)*eye(na+nb) (Fin=1000 by default)
%
% 		lam1 and lam0 make different adaptation algoritms as follows:
%		
%		lam1=1;lam0=1   	 :decreasing gain (default algorithm)
%		0.95<lam1<1;lam0=1 :decreasing gain with fixed forgetting factor 
%		0.95<lam1,lam0<1	 :decreasing gain with variable forgetting factor 
%
%		See also cloe, afcloe, xcloe, clvalid.
%		Copyright by Adaptech, 1997-1999



[nl,nc]=size(y);
if nc>2, error('This routine is only for SISO systems'),end
[nl,nc]=size(r);
if nc>2, error('This routine is only for SISO systems'),end
if (na<0 || nb<0 || d<0), error('The order of A,B and d should not be negative!!'),end

R=R./S(1);T=T./S(1);S=S./S(1);
[rs,cs]=size(S);
[rr,cr]=size(R);
[rt,ct]=size(T);
if min(size(R))>1, error('R should be a column vector!');end
if min(size(S))>1, error('S should be a column vector!');end
if min(size(T))>1, error('T should be a column vector!');end
if cs>rs, S=S';end
if cr>rr, R=R';end
if ct>rt, T=T';end

[ra0,ca0]=size(A0);
[rb0,cb0]=size(B0);
if ca0>ra0, A0=A0';end
if cb0>rb0, B0=B0';end

nd=min(length(r),length(y));	% number of data
nr=length(R);
ns=length(S);
nth=na+nb;

if nd<nth, error('Number of data should be greater than the number of parameters!'),end	

if nargin<10, error('This routin needs more parameters!'),end 
if nargin<11, lam1=1;lam0=1;Fin=1000;end
if nargin<12, lam1=1;lam0=1;end
if nargin<13, lam0=1;end

if isempty(Fin), Fin=1000;end
if isempty(lam1),lam1=1;end
if isempty(lam0), lam0=1;end

if (lam1>1 || lam0>1), error('lam1 and lam0 should be less than 1');end
if (lam1<0.95 || lam0<0.95), 
	disp ('warning :lam1 and lam0 are normally greater than 0.95');
end

nas=length(A0)-1+ns;
nbr=nr+length(B0)-1;
if nas>nbr, zas=0;zbr=nas-nbr;else zas=nbr-nas;zbr=0;end
P=[conv(A0,S);zeros(zas,1)]+[conv(B0,R);zeros(zbr,1)];
if max(abs(roots(P))) >= 1,
	error('Closed loop system with the initial model (B0,A0) should be stable!');
end

np0=length(P);
np=max([na+ns,nb+d+nr,np0]);
theta=zeros(nth,1);
phi=zeros(2*np,1);
phi_f=zeros(2*np,1);

nT=length(T);
rt=[zeros(nT+1,1)];

F=Fin*nth*eye(nth);

i=[1:np-1 np+1:2*np-1];		%shift index for vector phi
j=[1:na np+d+1:np+d+nb];  	%phi index for theta



for t=1:nd
	yhat=theta'*phi(j);
	e_apri=y(t)-yhat;
	e_apost=e_apri/(1+phi_f(j)'*F*phi_f(j));
	theta=theta+F*phi_f(j)*e_apost;

	F=1/lam1*(F-((F*phi_f(j)*phi_f(j)'*F)/(lam1+phi_f(j)'*F*phi_f(j))));
	lam1=lam0*lam1+1-lam0;

	yhat=theta'*phi(j);

	%Compute uhat by the control law
      rt(2:nT+1)=rt(1:nT);rt(1)=r(t);
	if ns>1, Su=S(2:ns)'*phi(np+1:np+ns-1);else Su=0;end
      uhat=R'*[-yhat;phi(1:nr-1)]-Su+T'*rt(1:nT);
   
	% Filter phi by S/P

	phi(i+1)=phi(i);phi(1)=-yhat;phi(np+1)=uhat;
	yuf=S'*[phi(1:ns) phi(np+1:np+ns)]-P(2:np0)'*[phi_f(1:np0-1) phi_f(np+1:np+np0-1)];
	phi_f(i+1)=phi_f(i);phi_f(1)=yuf(1);phi_f(np+1)=yuf(2);

end
A=[1;theta(1:na)]';
B=[zeros(d+1,1);theta(na+1:na+nb)]';
